searchData={"items":[{"type":"module","title":"Headless","doc":"","ref":"Headless.html"},{"type":"macro","title":"Headless.defc/2","doc":"Replace `def` with `defc` to optimize component rendering by inlining static attributes.","ref":"Headless.html#defc/2"},{"type":"macro","title":"Compile-time validation - Headless.defc/2","doc":"Using `defc` will also check in compile-time if the referenced attributes are correct.\n\nThis definition (using `def`) will compile and crash in runtime due to undefined `wrong` node reference.\n\n\n      def toggle(assigns) do\n        ~H\"\"\"\n        <.use_toggle :let={t}>\n          <.input {t.wrong}/>\n        </.use_toggle>\n        \"\"\"\n      end\n\n      # => ** (KeyError) key :wrong not found in: (...)\n\n\nWhen using `defc` it will not compile:\n\n\n      defc toggle(assigns) do\n        ~H\"\"\"\n        <.use_toggle :let={t}>\n          <.input {t.wrong}/>\n        </.use_toggle>\n        \"\"\"\n      end\n\n      # => ** (Phoenix.LiveView.Tokenizer.ParseError): Unknwon node \"wrong\" for component use_toggle","ref":"Headless.html#defc/2-compile-time-validation"},{"type":"macro","title":"Example - Headless.defc/2","doc":"defmodule ExampleAvatarComponents do\n      use Phoenix.Component\n      import Headless\n\n      attr :src, :any\n      attr :alt, :any\n      attr :initials, :string\n\n      defc avatar(assigns) do\n        ~H\"\"\"\n        <.use_avatar :let={a} src={@src}>\n           \n             \n             <%= @initials %> \n           \n        </.use_avatar>\n        \"\"\"\n      end\n    end","ref":"Headless.html#defc/2-example"},{"type":"function","title":"Headless.input/1","doc":"Render input for a form field.\n\nHeavily based on default Phoenix generated CoreComponents","ref":"Headless.html#input/1"},{"type":"function","title":"Text input - Headless.input/1","doc":"<.input field={@form[:name]} type=\"text\" placeholder=\"Name\"/>","ref":"Headless.html#input/1-text-input"},{"type":"function","title":"Checkbox - Headless.input/1","doc":"<.input field={@form[:is_admin]} type=\"checkbox\"/>","ref":"Headless.html#input/1-checkbox"},{"type":"function","title":"Attributes - Headless.input/1","doc":"* `field` (`Phoenix.HTML.FormField`) - a form field struct retrieved from the form, for example:  @form[:email] .\n* `id` (`:string`) - Defaults to `nil`.\n* `name` (`:string`)\n* `value` (`:string`)\n* `type` (`:string`) - Defaults to `\"text\"`.\n* `checked` (`:boolean`) - the checked flag for checkbox inputs.\n* Global attributes are accepted. additional attributes to be passed to the input element.","ref":"Headless.html#input/1-attributes"},{"type":"function","title":"Slots - Headless.input/1","doc":"* `inner_block`","ref":"Headless.html#input/1-slots"},{"type":"function","title":"Headless.input_error/1","doc":"Render input errors\n\nHeavily based on default Phoenix generated CoreComponents\n\nStorybook: No errors\n    <.input_error field={@form[:name]} :let={{msg, _opts}}>\n      <%= msg %>\n    </.input_error>\n\nStorybook: With errors\n    <.input_error field={@form[:email]} :let={{msg, _opts}}>\n      <%= msg %>\n    </.input_error>","ref":"Headless.html#input_error/1"},{"type":"function","title":"Attributes - Headless.input_error/1","doc":"* `field` (`Phoenix.HTML.FormField`) (required) - a form field struct retrieved from the form, for example:  @form[:email] .","ref":"Headless.html#input_error/1-attributes"},{"type":"function","title":"Slots - Headless.input_error/1","doc":"* `inner_block` (required) - The default slot will be rendered with  {message, options}  tuple.","ref":"Headless.html#input_error/1-slots"},{"type":"function","title":"Headless.use_avatar/1","doc":"Avatar component","ref":"Headless.html#use_avatar/1"},{"type":"function","title":"Elements - Headless.use_avatar/1","doc":"- `root` - root element\n- `image` - ` ` element\n- `fallback` - fallback element","ref":"Headless.html#use_avatar/1-elements"},{"type":"function","title":"Example - Headless.use_avatar/1","doc":"defmodule ExampleAvatarComponents do\n      use Phoenix.Component\n      import Headless\n\n      attr :src, :any\n      attr :alt, :any\n      attr :initials, :string\n\n      def avatar(assigns) do\n        ~H\"\"\"\n        <.use_avatar :let={a} src={@src}>\n           \n             \n             <%= @initials %> \n           \n        </.use_avatar>\n        \"\"\"\n      end\n\n      def example(assigns) do\n        ~H\"\"\"\n        <.avatar src=\"https://example.com/avatar.jpg\" alt=\"Avatar\" initials=\"JD\" />\n        <.avatar src={nil} alt=\"Avatar\" initials=\"JD\" />\n        \"\"\"\n      end\n    end","ref":"Headless.html#use_avatar/1-example"},{"type":"function","title":"Attributes - Headless.use_avatar/1","doc":"* `src` (`:string`) - Image source URL.","ref":"Headless.html#use_avatar/1-attributes"},{"type":"function","title":"Slots - Headless.use_avatar/1","doc":"* `inner_block` (required)","ref":"Headless.html#use_avatar/1-slots"},{"type":"function","title":"Headless.use_clipboard/1","doc":"Clipboard component.\n\nProvides copy to clipboard functionality.","ref":"Headless.html#use_clipboard/1"},{"type":"function","title":"Elements - Headless.use_clipboard/1","doc":"- `root` - root element\n- `trigger` - element that triggers the clipboard action\n- `content` - element that contains the clipboard content","ref":"Headless.html#use_clipboard/1-elements"},{"type":"function","title":"Example - Headless.use_clipboard/1","doc":"defmodule ExampleClipboardComponents do\n      use Phoenix.Component\n      import Headless\n\n      attr :text, :string\n\n      def copy_to_clipboard(assigns) do\n        ~H\"\"\"\n        <.use_clipboard :let={c}>\n           \n             \n             Copy \n           \n        </.use_clipboard>\n        \"\"\"\n      end\n\n      def example(assigns) do\n        ~H\"\"\"\n        <.copy_to_clipboard text=\"Secrets\"/>\n        \"\"\"\n      end\n    end","ref":"Headless.html#use_clipboard/1-example"},{"type":"function","title":"Slots - Headless.use_clipboard/1","doc":"* `inner_block` (required)","ref":"Headless.html#use_clipboard/1-slots"},{"type":"function","title":"Headless.use_popover/1","doc":"Popover component","ref":"Headless.html#use_popover/1"},{"type":"function","title":"Elements - Headless.use_popover/1","doc":"- `root` - root element\n- `trigger` - element that triggers the popover\n- `content` - element that contains the popover content","ref":"Headless.html#use_popover/1-elements"},{"type":"function","title":"Example - Headless.use_popover/1","doc":"defmodule ExamplePopoverComponents do\n      use Phoenix.Component\n      import Headless\n\n      slot :trigger\n      slot :content\n\n      def popover(assigns) do\n        ~H\"\"\"\n        <.use_popover :let={p}>\n           \n             \n              <%= render_slot(@trigger) %>\n             \n             \n              <%= render_slot(@content) %>\n             \n           \n        </.use_popover>\n        \"\"\"\n      end\n\n      def example(assigns) do\n        ~H\"\"\"\n        <.popover>\n          <:trigger>\n            Open\n          </:trigger>\n          <:content>\n            Hidden message\n          </:content>\n        </.popover>\n        \"\"\"\n      end\n    end","ref":"Headless.html#use_popover/1-example"},{"type":"function","title":"Slots - Headless.use_popover/1","doc":"* `inner_block` (required)","ref":"Headless.html#use_popover/1-slots"},{"type":"function","title":"Headless.use_toggle/1","doc":"Toggle (switch) component.\n\nUses regular checkbox input under the hood.","ref":"Headless.html#use_toggle/1"},{"type":"function","title":"Elements - Headless.use_toggle/1","doc":"- `input` - the underlying checkbox element","ref":"Headless.html#use_toggle/1-elements"},{"type":"function","title":"Example - Headless.use_toggle/1","doc":"defmodule ExampleToggleComponents do\n      use Phoenix.Component\n      import Headless\n\n      def toggle(assigns) do\n        ~H\"\"\"\n        <.use_toggle :let={t}>\n          <.input {t.input} field={@field}/>\n        </.use_toggle>\n        \"\"\"\n      end\n\n      def example(assigns) do\n        ~H\"\"\"\n        <.toggle field={@form[:is_admin]}/>\n        \"\"\"\n      end\n    end","ref":"Headless.html#use_toggle/1-example"},{"type":"function","title":"Slots - Headless.use_toggle/1","doc":"* `inner_block` (required)","ref":"Headless.html#use_toggle/1-slots"},{"type":"extras","title":"Headless UI Components for Phoenix","doc":"# Headless UI Components for Phoenix\n\nUnstyled, accessible UI components for Phoenix and Phoenix LiveView.\nTo be styled with the CSS framework of your choice.\n\n> [!WARNING]\n> This project is in a very early stage - see [Components](#components) list below.\n\n![Preview](preview.png)","ref":"readme.html"},{"type":"extras","title":"Demo - Headless UI Components for Phoenix","doc":"[See demo website](https://headless.fly.dev)","ref":"readme.html#demo"},{"type":"extras","title":"Goals & Rules - Headless UI Components for Phoenix","doc":"- Provide unstyled Phoenix components as building blocks for your own UI components\n- If something can be achieved with HTML and CSS only, it should be done with HTML and CSS only (no-JS)\n- Where JS is required, use [Alpine.js](http://alpinejs.dev)\n- Use [`Alpine.data()`](https://alpinejs.dev/globals/alpine-data) instead of inline markup\n- Components must work with standard Phoenix controllers (_dead_ views)\n- Components must work with Phoenix LiveView\n- Components must work with standard Phoenix forms\n- Components must be accessible (aria attributes, keyboard navigation, focus, etc.)","ref":"readme.html#goals-rules"},{"type":"extras","title":"Components - Headless UI Components for Phoenix","doc":"| Component    | Functions         | Status         |\n| ------------ | ----------------- | -------------- |\n| Avatar       | `use_avatar/1`    | ✅ Done        |\n| Checkbox     | `input/1`         | ✅ Done        |\n| Clipboard    | `use_clipboard/1` | ✅ Done        |\n| Combobox     | `use_combobox/1`  | 🏗️ In progress |\n| Command      |                   | 🏗️ In progress |\n| Dialog       |                   | 🗺️ Planned     |\n| File Preview |                   | 🗺️ Planned     |\n| Input OTP    |                   | 🗺️ Planned     |\n| Popover      | `use_popover/1`   | ✅ Done        |\n| Radio button |                   | 🗺️ Planned     |\n| Tabs         |                   | 🗺️ Planned     |\n| Text input   | `input/1`         | ✅ Done        |\n| Textarea     |                   | 🗺️ Planned     |\n| Toggle       | `use_toggle/1`    | ✅ Done        |","ref":"readme.html#components"},{"type":"extras","title":"Installation - Headless UI Components for Phoenix","doc":"The package can be installed by adding `headless` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:headless, \"~> 0.1\"}\n  ]\nend\n```\n\nInclude JavaScript package in your `app.js`:\n\n```js\n// assets/js/app.js\n\n// import and start headless\nimport headless from \"headless\"\nheadless.start()\n\n// ...\n\n// configure LiveSocket\nlet liveSocket = new LiveSocket('/live', Socket, {\n  // ...\n\n  // configure dom hook\n  dom: headless.dom\n})\n```","ref":"readme.html#installation"},{"type":"extras","title":"Usage - Headless UI Components for Phoenix","doc":"Headless components are meant to be used as building blocks for your own components.\nMost components are built using `use_*` functions that expose the necessary HTML attributes\nto provide the functionality leaving all tag rendering to the user.\nThis way every element can be 100% customized.\n\nSee [example app components](./apps/demo/lib/demo/components/core_components.ex).\n\n```elixir\ndefmodule MyAppWeb.Components do\n  use Phoenix.Component\n  import Headless\n\n  attr :src, :any\n  attr :alt, :any\n  attr :initials, :string\n\n  def avatar(assigns) do\n    ~H\"\"\"\n    <.use_avatar :let={a} src={@src}>\n       \n         \n         <%= @initials %> \n       \n    </.use_avatar>\n    \"\"\"\n  end\nend\n```","ref":"readme.html#usage"},{"type":"extras","title":"Adding your own Alpine components - Headless UI Components for Phoenix","doc":"If you want to add your own Alpine components you can import the bundled Alpine like this:\n\n```js\n// assets/js/app.js\n\nimport headless, { Alpine } from \"headless\"\n\nAlpine.data(\"my_custom_component\", () => ...)\n\nheadless.start()\n```","ref":"readme.html#adding-your-own-alpine-components"},{"type":"extras","title":"Development - Headless UI Components for Phoenix","doc":"```bash\n# Start development server with examples\nmix phx.server\n\n# Update bundled Alpine\ncurl -L https://unpkg.com/@alpinejs/csp/dist/module.cjs.js > ./apps/headless/assets/vendor/alpine.js\n```","ref":"readme.html#development"},{"type":"extras","title":"Inspirations - Headless UI Components for Phoenix","doc":"- [headless UI](https://headlessui.com)\n- [Sprout UI](https://github.com/TunkShif/sprout_ui)\n- [Zig](https://zagjs.com/)","ref":"readme.html#inspirations"},{"type":"extras","title":"LICENSE","doc":"The MIT License (MIT)\n\nCopyright (c) 2024 Tymon Tobolski\n\nPermission is hereby granted, free of charge, to any person obtaining a copy\nof this software and associated documentation files (the \"Software\"), to deal\nin the Software without restriction, including without limitation the rights\nto use, copy, modify, merge, publish, distribute, sublicense, and/or sell\ncopies of the Software, and to permit persons to whom the Software is\nfurnished to do so, subject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in all\ncopies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\nIMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\nFITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\nAUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\nLIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,\nOUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE\nSOFTWARE.","ref":"license.html"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,50,46,50]}}